import java.io.*;
import java.util.*;
import java.text.*;

/**
 * manages the reading and wrinting of the isotope file
 * 
 * @author Team NAH
 */
public class FileIO {

	private String file;

	/**
	 * Constructor
	 * 
	 * @param file
	 *            address of file to read
	 */
	public FileIO(String file) {
		this.file = file;
	}

	// ////////////BEGIN: READ CODE/////////////////////////
	/**
	 * Creates a <code>FileInputStream</code> object and pass it to the
	 * parser, then process the parse tree to turn it into a useful data
	 * structure.
	 * 
	 * @return the parsed isotope file in a <code>Hashtable</code>
	 */
	public Hashtable readFile() {
		Hashtable ht = null;

		try {
			FileInputStream fs = new FileInputStream(file);
			FileParser fp = new FileParser(fs);
			ASTInput jjtree = fp.Input();
			ht = processJJTree(jjtree);
		} catch (FileNotFoundException e) {
			ErrorSubject.quickError(0, "The isotope file '" + file
					+ "' was not found");
			return null;
		} catch (ParseException e) {
			ErrorSubject
					.quickError(0, "Isotope file '" + file + "' is corrupt");
			return null;
		} catch (Exception e) {
			ErrorSubject.quickError(0,
					"An error occoured while reading the file: "
							+ e.getMessage());
			return null;
		}

		return ht;
	}

	/**
	 * Adds each <code>ASTElement</code> in <code>tree</code> into the
	 * <code>Hashtable</code>.
	 * 
	 * @param tree
	 *            The root of the tree
	 * @return a <code>Hashtable</code> instantiated.
	 */
	private Hashtable processJJTree(ASTInput tree) {
		Hashtable ht = new Hashtable(50);
		int i, count;
		Class element_node = ASTElement.class;

		count = tree.jjtGetNumChildren();
		for (i = 0; i < count; i++) {
			Node node = tree.jjtGetChild(i);
			if (node instanceof ASTElement) {
				ht.put(((Element) node).getSymbol(), node);
				processASTElement((ASTElement) node);
			}
		}

		return ht;
	}

	/**
	 * Adds each isotope to an <code>Element</code>.
	 * 
	 * @param tree
	 *            is an <code>ASTElement</code>
	 */
	private void processASTElement(ASTElement tree) {
		int count = tree.jjtGetNumChildren();
		Class isotope_node = ASTIsotope.class;

		for (int i = 0; i < count; i++) {
			Node node = tree.jjtGetChild(i);
			if (node.getClass().equals(isotope_node)) {
				tree.getIsotopes().add((Isotope) node);
			}
		}
	}

	// //////////////END: READ CODE/////////////////////////

	// ////////////BEGIN: WRITE CODE////////////////////////
	/**
	 * <p>
	 * Write <code>isotopes</code> to the file
	 * </p>
	 * 
	 * @param isotopes
	 *            an array of Element objects to be written to the file
	 */
	public boolean writeFile(Hashtable isotopes) {
		BufferedWriter bw = null;
		try {
			bw = new BufferedWriter(new FileWriter(file));
			writeElements(isotopes, bw);
		} catch (FileNotFoundException e) {
			ErrorSubject.quickError(0,
					"The file selected was a directory or not accessable");
			return false;
		} catch (SecurityException e) {
			ErrorSubject.quickError(0,
					"Can not write the file, not sufficent access privileges");
			return false;
		} catch (IOException e) {
			ErrorSubject.quickError(0,
					"An I/O error occoured while trying to write isotope file");
			try {
				bw.close();
			} catch (Exception ex) {
			}
			return false;
		} catch (Exception e) {
			ErrorSubject
					.quickError(0,
							"An error occoured while trying to write isotopes to the file");
			ErrorSubject.quickError(0, e.getMessage());
			try {
				bw.close();
			} catch (Exception ex) {
			}
			return false;
		}

		try {
			bw.close();
		} catch (IOException ex) {
			ErrorSubject
					.quickError(0,
							"There was an error closing the output stream after writing the isotope file");
			return false;
		}

		return true;
	}

	/**
	 * Enumerates the <code>elemetns</code> looking for <code>Element</code>
	 * objects, when found calls <code>writeElement</code> passing it in.
	 * 
	 * @param elements
	 *            a hashtable of the elemetns to write
	 * @param wr
	 *            a writer to write the data to
	 * @return the number of elements written
	 */
	private int writeElements(Hashtable elements, Writer wr) throws Exception {
		Enumeration enumeration = elements.elements();
		int counter = 0;

		while (enumeration.hasMoreElements()) {
			Object obj = enumeration.nextElement();
			if (obj instanceof Element) {
				if (writeElement((Element) obj, wr))
					counter += 1;
			} else
				ErrorSubject
						.quickError(
								0,
								"While trying to write the isotope file a bad element was found in the hashtable, skipping element");
		}
		return counter;
	}

	/**
	 * <p>
	 * Writes the record header for the element
	 * </p>
	 * e.g. <code>Symbol:Name:AtomicNumber</code>
	 * <p>
	 * then calls <code>writeIsotope</code> to write the isotopes to the
	 * stream.
	 * 
	 * @param e
	 *            an element to write to the stream
	 * @param wr
	 *            a <code>Writer</code>, probably a
	 *            <code>BufferedWriter</code> attached to a
	 *            <code>FileOutputStream</code>
	 */
	private boolean writeElement(Element e, Writer wr) throws Exception {
		char[] tmp = e.getSymbol().toCharArray();
		for (int i = 0; i < tmp.length; i++) {
			if (((i == 0) && (!Character.isUpperCase(tmp[i])))
					|| ((i != 0) && (!Character.isLowerCase(tmp[i])))) {
				ErrorSubject.quickError(0, "A badly formatted symbol '"
						+ e.getSymbol()
						+ "' was found while trying to write an element");
				/*
				 * return false to indicate that the symbol format was bad. If
				 * written as it is then the parser will not be able to parse
				 * the file.
				 */
				return false;
			}
		}

		wr.write(e.getSymbol() + ":" + e.getName().toLowerCase() + ":"
				+ e.getAtomicNumber());
		int counter = 0;
		for (int i = 0; i < e.getIsotopes().count(); i++) {
			if (writeIsotope(e.getIsotopes().getIsotope(i), wr))
				counter++;
		}
		wr.write("\n");

		if (counter > 0)
			return true;

		throw new Exception(
				"A major error occoured while writing the isotope file, an element was written with no isotopes. The parser will not be able to read this file. It is corrupt.");
	}

	/**
	 * Writes an isotpe to the writer, uses a <code>NumberFormat</code> object
	 * to format the decimal numbers to get rid of expnential notation and make
	 * sure that the number has a decimal point.
	 * 
	 * @param iso
	 * @param wr
	 * @return
	 */
	private boolean writeIsotope(Isotope iso, Writer wr) throws IOException {
		NumberFormat nf = NumberFormat.getInstance();
		if (nf instanceof DecimalFormat) {
			DecimalFormat df;
			df = (DecimalFormat) nf;
			df.applyPattern("##0.0#########");
		}
		wr.write(":(" + nf.format(iso.getAtomicMass()) + ":"
				+ nf.format(iso.getPercent()) + ")");
		return true;
	}
	// //////////////END: WRITE CODE////////////////////////
}
