import java.util.*;

/**
 * <p>
 * A simple element representation, used when a read / write access is needed to
 * an element.
 * </p>
 * 
 * <p>
 * performs validation on new data assigned to the element
 * </p>
 * 
 * @author Team NAH
 */
public class SimpleElement implements Element {
	private String symbol = "", name = "";

	private int number;

	private IsotopeList isotopes = new IsotopeList(3);

	public String getSymbol() {
		return symbol;
	}

	public String getName() {
		return name;
	}

	public int getAtomicNumber() {
		return number;
	}

	public IsotopeList getIsotopes() {
		return isotopes;
	}

	/**
	 * sets the symbol for this element
	 * 
	 * @param val
	 *            a Symbol, doesnt need to be specialy formatted because passed
	 *            to <tt>titleCase</tt>
	 * @return true if a valid symbol was entered, false if the symbol existed
	 *         in the library.
	 */
	public boolean setSymbol(String val) {
		String tmp = titleCase(val);
		if (IsotopeLibrary.getLibrary().getElement(tmp) != null)
			return false;
		symbol = tmp;
		return true;
	}

	/**
	 * Simple set method, doesnt need to enforce any data validation. Converts
	 * the name to title case.
	 * 
	 * @param val
	 */
	public void setName(String val) {
		name = titleCase(val);
	}

	/**
	 * sets the atomic number of this element, validates the number as well.
	 * 
	 * @param val
	 *            a new atomic number, must be unique.
	 * @return true if the value was set, false if the atomic number is not
	 *         unique in the library
	 */
	public boolean setAtomicNumber(int val) {
		Enumeration enumerator = IsotopeLibrary.getLibrary().getEnumeration();
		while (enumerator.hasMoreElements()) {
			Object obj = enumerator.nextElement();
			if (obj instanceof Element)
				if (((Element) obj).getAtomicNumber() == val)
					return false;
		}
		number = val;
		return true;
	}

	/**
	 * converts a string into title case
	 * 
	 * @param str
	 *            a string to fromat
	 * @return <tt>str</tt> converted to title case
	 */
	private String titleCase(String str) {
		if (str.equals(""))
			return str;
		char[] tmp = str.toLowerCase().toCharArray();
		tmp[0] = Character.toUpperCase(tmp[0]);
		int i;
		for (i = 0; i < tmp.length; i++) {
			if (!Character.isLetter(tmp[i]))
				break;
		}
		return new String(tmp, 0, i);
	}

	/**
	 * validates this <tt>Element</tt>, returns a value according to the
	 * validitiy of the record.
	 * 
	 * @throws <tt>IllegalArgumentException</tt> indicating one of the
	 *             attributes is invalid
	 */
	public void validate() throws IllegalArgumentException {
		if (symbol.equals(""))
			throw new IllegalArgumentException(
					"The symbol attribute of the new element failed validation, cant be empty");
		if (name.equals(""))
			throw new IllegalArgumentException(
					"The name attribute of the new element failed validation, can't be empty");
		if (number < 1)
			throw new IllegalArgumentException(
					"The 'atomic number' attribute of the new element failed validation, cant be zero");
		if (isotopes.count() < 1)
			throw new IllegalArgumentException(
					"The isotope list failed validation, can't be empty");
	}
}
