import java.util.*;
import java.io.*;

/**
 * Isotope Library
 * 
 * manages the Isotopes; keeps all the elements in a hashtable indexed by their
 * elemental symbol. Each element from the hashtable is a vector of the isotopes
 * that element contains. Each isotope is made from two doubles.
 * 
 * @author Team NAH
 * @version Spring 2006
 */
public class IsotopeLibrary {

	// //////////////BEGIN: SINGLETON CODE////////////////////

	// unique static reference to the engine
	private static IsotopeLibrary ref;

	private IsotopeLibrary() {
	}

	/**
	 * Get a unique instance of the engine object.
	 * 
	 * Returns the unique instance of engine (creates it if it doesn't already
	 * exist.)
	 */
	public static IsotopeLibrary getLibrary() {
		if (ref == null) {
			ref = new IsotopeLibrary();
		}
		return ref;
	}

	/**
	 * Intercepts any attempts to clone the object.
	 * 
	 * This method exploits polymorphism by intercepting attempts to clone the
	 * object and throwing an exception. By doing this, the class is guaranteed
	 * not to be duplicated.
	 */
	public Object clone() throws CloneNotSupportedException {
		throw new CloneNotSupportedException();
	}

	// //////////////END: SINGLETON CODE////////////////////

	// ////////////BEGIN: LIBRARY IO CODE///////////////////

	private Hashtable isotopes;

	/**
	 * Initilises the library, reads the file and converts the parsed jjtree
	 * into a list of isotopes.
	 */
	public boolean initilise(String file) {
		FileIO f = new FileIO(file);
		isotopes = f.readFile();

		if (isotopes == null) {
			ErrorSubject
					.quickError(
							0,
							"The isotope library could not initialise, an error has occoured while using file '"
									+ file + "'");
			isotopes = new Hashtable();
			return false;
		}
		return true;
	}

	/**
	 * saves the contents of the hashtable
	 * 
	 * @param file
	 *            location to save the isotopes to
	 * @return true indicates success
	 */
	public boolean save_library(String file) {
		try {
			FileIO f = new FileIO(file);
			f.writeFile(isotopes);
		} catch (Exception e) {
			ErrorSubject.quickError(0,
					"There has been an error writing the isotope file '" + file
							+ "'");
			return false;
		}
		return true;
	}

	// //////////////END: LIBRARY IO CODE///////////////////

	// ////////////BEGIN: LIBRARY CODE//////////////////////
	/**
	 * @param symbol
	 *            Chemical formula of the element wanted
	 * @return The <code>Element</code> of the passed in symbol
	 */
	public Element getElement(String symbol) {
		return (Element) isotopes.get(symbol);
	}

	/**
	 * Returns the number of <code>Element</code> objects in the hashtable
	 */
	public int count() {
		int counter = 0;
		Enumeration enumerator = isotopes.elements();
		while (enumerator.hasMoreElements()) {
			Object obj = enumerator.nextElement();
			if (obj instanceof Element)
				counter++;
		}
		return counter;
	}

	/**
	 * @return a collection that has all the elemements from the hashtable
	 * @see java.util.Hashtable.values()
	 */
	public Collection toCollection() {
		return isotopes.values();
	}

	/**
	 * @return an enumeration for the underlaying Hashtable
	 */
	public Enumeration getEnumeration() {
		return isotopes.elements();
	}

	/**
	 * adds a new element to the library
	 * 
	 * @param e
	 * @return true indicates success
	 */
	public boolean addElement(Element e) {
		if (getElement(e.getSymbol()) == null) {
			isotopes.put(e.getSymbol(), e);
			return true;
		}
		return false;
	}

	/**
	 * removes an element from the library
	 * 
	 * @param e
	 * @return true indicates success
	 */
	public boolean removeElement(String symbol) {
		if (getElement(symbol) != null) {
			isotopes.remove(symbol);
			return true;
		}
		return false;
	}
	// //////////////END: LIBRARY CODE//////////////////////
}
